// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>
#include <sys/types.h>
#include <unistd.h>

void gen_dos_fd_n()
{
printf("Electrons....\n");
FILE *out;
double m=0.0;
double xstart=0.0;
double xsteps=0.0;
double dx=0.0;
double tstart=0.0;
double tstop=0.0;
double tsteps=0.0;
double dt=0.0;
double xpos=0.0;
double tpos=0.0;
int t=0;
int x=0;

m=9e-1;
out=fopen("./dosn_fd.dat","w");
xstart=-4.0;
xsteps=700;
dx=1e-3;
tstart=280;
tstop=340;
tsteps=1000;
dt=(tstop-tstart)/tsteps;
xpos=xstart;
tpos=tstart;
t=0;
x=0;
double 	dx0=1.0e-3;
fprintf(out,"%d %d\n",(int)xsteps,(int)tsteps);

xpos=xstart;
double *xmesh=NULL;
xmesh=(double *)malloc(sizeof(double)*xsteps);
for (x=0;x<xsteps;x++)
{
	dx=dx0;
	if (xpos<-0.4) dx=dx0*3.0;
	if (xpos<-1.0) dx=dx0*10.0;
	fprintf(out,"%e\n",xpos);
	xmesh[x]=xpos;
	xpos+=dx;
}

for (t=0;t<tsteps;t++)
{
	fprintf(out,"%e\n",tpos);
	tpos+=dt;
}
int e=0;
double Esteps=1100;
double Estart=0.0;
double Estop=4.0*q;
double dE=(Estop-Estart)/Esteps;
double E=Estart;
double rho;
double sum=0.0;
double f=0.0;
tpos=tstart;
double sum_w=0.0;
//#define test_dist_n
#ifdef test_dist_n
tpos=300;
FILE *test;
test=fopen("./test.dat","w");
#endif
for (t=0;t<tsteps;t++)
{
printf("Electron %d/%d\n",t,(int)tsteps);

	for (x=0;x<xsteps;x++)
	{
		xpos=xmesh[x];
		E=Estart;
		sum=0.0;
		sum_w=0.0;
		for (e=0;e<(Esteps);e++)
		{
			rho=(sqrt(E)/(4.0*pi*pi))*pow((2.0*m*m0)/(hbar*hbar),3.0/2.0);
			f=1.0/(1.0+exp((-xpos+E/q)*q/(kb*tpos)));
			sum+=2.0*rho*f*dE;
			sum_w+=E*2.0*rho*f*dE;

			//printf("%e %e %e %e %e",E/q,rho,f,dE,sum);
			//getchar();
			E+=dE;
		}
		//N=2.0*pow(((2.0*pi*kb*tpos*m*m0)/(hp*hp)),1.5);
		//p=N*exp((xpos*q)/(kb*tpos));
		fprintf(out,"%.20le %.20le\n",sum,sum_w/sum);
		#ifdef test_dist_n
		fprintf(test,"%.20le %.20le %.20le\n",xpos,sum_w/sum/q,(3.0/2.0)*kb*tpos/q);
		#endif
		//printf("%.20le %.20le\n",sum_w/sum,(3.0/2.0)*kb*tpos);
		//getchar();
	}

#ifdef test_dist_n
fclose(test);
exit(0);
#endif
	tpos+=dt;
}

fclose(out);
free(xmesh);
return;
}


void gen_dos_fd_p()
{
printf("Holes....\n");
FILE *out;
double m=0.0;
double xstart=0.0;
double xsteps=0.0;
double dx=0.0;
double tstart=0.0;
double tstop=0.0;
double tsteps=0.0;
double dt=0.0;
double xpos=0.0;
double tpos=0.0;
int t=0;
int x=0;
double N=0.0;
double p=0.0;

int e=0;
double Esteps=4000;
double Estart=0.0;
double Estop=4.0*q;
double dE=(Estop-Estart)/Esteps;
double E=Estart;
double rho;
double sum=0.0;
double f=0.0;
double *xmesh;
double dx0=3e-3;
double sum_w=0.0;



m=9e-1;
out=fopen("./dosp_fd.dat","w");
xstart=-2.0;
xsteps=300;
tstart=290;
tstop=300;
tsteps=1000;
dt=(tstop-tstart)/tsteps;
xpos=xstart;
tpos=tstart;
t=0;
x=0;

fprintf(out,"%d %d\n",(int)xsteps,(int)tsteps);

xmesh=(double *)malloc(sizeof(double)*xsteps);
for (x=0;x<xsteps;x++)
{
	dx=dx0;
	if (xpos<-0.4) dx=dx0*3.0;
	if (xpos<-1.0) dx=dx0*10.0;
	fprintf(out,"%e\n",xpos);
	xmesh[x]=xpos;
	xpos+=dx;
}

for (t=0;t<tsteps;t++)
{
	fprintf(out,"%e\n",tpos);
	tpos+=dt;
}

tpos=tstart;

//#define test_dist_p
#ifdef test_dist_p
tpos=300;

FILE *test;
test=fopen("./test.dat","w");
#endif

for (t=0;t<tsteps;t++)
{
printf("Hole %d/%d\n",t,(int)tsteps);

	for (x=0;x<xsteps;x++)
	{
		xpos=xmesh[x];
		E=0.0;
		sum=0.0;
		sum_w=0.0;
		for (e=0;e<(Esteps);e++)
		{
			rho=(sqrt(E)/(4.0*pi*pi))*pow((2.0*m*m0)/(hbar*hbar),3.0/2.0);
			f=1.0/(1.0+exp((-xpos+E/q)*q/(kb*tpos)));
			//printf("%e %e %e\n",xpos+E/q,dE/q,E/q);
			//getchar();
			sum+=2.0*rho*f*dE;
			sum_w+=E*2.0*rho*f*dE;
			E+=dE;
		}

		//N=2.0*pow(((2.0*pi*kb*tpos*m*m0)/(hp*hp)),1.5);
		//p=N*exp((xpos*q)/(kb*tpos));

		N=2.0*pow(((2.0*pi*kb*tpos*m*m0)/(hp*hp)),1.5);
		p=N*exp((xpos*q)/(kb*tpos));

		fprintf(out,"%.20le %.20le\n",sum,sum_w/sum);

		#ifdef test_dist_p
		fprintf(test,"%.20le %.20le %.20le\n",xpos,sum_w/sum/q,(3.0/2.0)*kb*tpos/q);
		#endif
		//printf("%le %le\n",p,sum);
		//getchar();
		//xpos+=dx;
	}

	#ifdef test_dist_p
	fclose(test);
	exit(0);
	#endif
	tpos+=dt;
}

free(xmesh);
fclose(out);

}


void gen_dos_fd()
{
pid_t pID = fork();
if (pID == 0)
{
gen_dos_fd_n();
exit(0);
}else
{
gen_dos_fd_p();
}


printf("Finished generating....\n");
}


