// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

#ifdef makegui
#include <gtk/gtk.h>
#include "gui.h"
#endif


struct device cell;

void printdebug(char *text)
{
#ifdef makegui
//printf("'%s'\n",gui.filename);
//getchar();
if (gui.show==TRUE)
{
  GtkTextBuffer *tbuffer;
  GtkTextIter ei;

  tbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(gui.textview));
  gtk_text_buffer_get_end_iter(tbuffer, &ei);
  gtk_text_buffer_insert(tbuffer, &ei, text, -1);
gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (gui.textview),
                                      &ei, 0.0, FALSE, 0, 0);
}else
{
#endif
printf("%s",text);
#ifdef makegui
}
#endif
}


#ifdef makegui
struct gui gui;

struct gui* get_gui()
{
return &gui;
}



void about_box_show(GtkWidget *widget, gpointer user_data)
{

const gchar* authors[] = {
		"Roderick MacKenzie <rod@roderickmackenzie.eu> or <r.c.i.mackenzie@googlemail.com> or <r.mackenzie@imperial.ac.uk>",
		NULL
    };

	GdkPixbuf *logo = gdk_pixbuf_new_from_file ("sun.png", NULL);

	gtk_show_about_dialog(NULL,
	                     "name", "Solarsim",
			     "program-name" ,"Solarsim",
	                     "version", "V 0.1",
	                     "copyright", "Copyright \xc2\xa9 2008 Roderick MacKenzie",
	                     "comments",  "Organic solar cell simulator",
	                     "website", "<www.roderickmackenzie.eu>",
	                     "authors", authors,
	                     "license", "SolarSim V0.1 License:\n This software is not open source and the source\ncode or the executable may not be distributed.   I do not mind you using\n this code for your academic research. However, due to the large amount of\n time I have invested in the software I must insist that I am included as\n an author on any resulting publications.  rod@roderickmackenzie.eu",
	                  	"logo", logo,
	                     NULL);
}








void warn_dialog(gchar *message,char* type)
{
  gchar *markup;
  GtkDialog *dialog;
  GtkHBox *hbox;
  GtkImage *icon;
  GtkLabel *text;
//  GtkWidget *mainwindow;
  markup = g_strdup_printf("%s",
			   message);
	//mainwindow = glade_xml_get_widget (glade_xml, "MainWindow");

  dialog = GTK_DIALOG(gtk_dialog_new_with_buttons(
                      "Solar sim", NULL, GTK_DIALOG_MODAL,
                      "OK", GTK_RESPONSE_OK, NULL));

  gtk_dialog_set_default_response(dialog, GTK_RESPONSE_OK);
  g_signal_connect_swapped(dialog, "response", G_CALLBACK(gtk_widget_destroy),
                           GTK_WIDGET(dialog));

  hbox = (GtkHBox*)g_object_new(GTK_TYPE_HBOX, "border-width", 8, NULL);
  icon = (GtkImage*)g_object_new(GTK_TYPE_IMAGE,
		      "stock", type,
                      "icon-size", GTK_ICON_SIZE_DIALOG,
                      "xalign", 0.5, "yalign", 0.5, NULL);

  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(icon), FALSE, FALSE, 0);
  text = (GtkLabel*)g_object_new(GTK_TYPE_LABEL,
                      "wrap", TRUE,
                      "label", markup,
                      "use-markup", TRUE, NULL);
  g_free(markup);

  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(text), TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(dialog->vbox), GTK_WIDGET(hbox),
                     FALSE, FALSE, 0);
  gtk_widget_show_all(GTK_WIDGET(dialog));
}




double get_error(struct device *in,double *b)
{
double tot=0.0;
int i;
for (i=0;i<in->ymeshpoints;i++)
{

	tot+=mod(b[i]);
}
return tot;
}



void on_window_destroy (GtkObject *object, gpointer user_data)
{
    gtk_main_quit ();
}

void on_simulation_run (GtkObject *object, gpointer user_data)
{
printdebug( "Running simulation\n");
cell.stop=FALSE;
cell.onlypos=FALSE;

    run_simulation(gui.filename);
warn_dialog("<big>Done!</big>\n\nThe simulation has finished.",GTK_STOCK_DIALOG_INFO);
}


void equ_click (GtkObject *object, gpointer user_data)
{
printdebug( "Calculating equilibrium\n");
cell.stop=FALSE;
cell.onlypos=TRUE;

    run_simulation(gui.filename);

FILE *plot = popen("gnuplot -persist","w");
fprintf(plot, "set terminal x11 title 'Solarsim - equilibrium band structure'\n");
fprintf(plot, "clear\n");
fprintf(plot, "set data style lines\n");
fprintf(plot, "load 'equ.plot'\n");
fflush(plot);
fclose(plot);
}

void on_show_configure_click (GtkObject *object, gpointer user_data)
{
GtkWidget * widget;
	gtk_widget_show(gui.configure);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "ex1"));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.enable_ex);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "plotresults"));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "voltageorlight"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.pulsevoltage);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "plotresults"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.enable_plot);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "eb1"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.nnewton);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "hb1"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.hnewton);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pb1"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.lnewton);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "timedomain"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.time_on);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "psun"));

	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Psun);//
printf("%lf\n",gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget)));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "startvoltage"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Vstart);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "stopvoltage"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Vstop);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "stepvoltage"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Vstep);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "ddclamp"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.electrical_clamp);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "eclamp"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.thermal_clamp);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "hclamp"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.thermal_clamp);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "dt"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.dt/1e-9);


	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "onvalue"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.onvalue);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "offvalue"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.offvalue);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pulsestart"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.pulsestart);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "reprate"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.reprate);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pulselength"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.pulselen);


	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "electricalerror"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), fabs(log10(gui_device.min_cur_error)));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "tleft"));

	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), 300);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "tr"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Tlr);


}


void on_configure_ok (GtkObject *object, gpointer user_data)
{
GtkWidget * widget;
	gtk_widget_show(gui.configure);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "ex1"));
	gui_device.enable_ex = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "plotresults"));
	gui_device.enable_plot = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "eb1"));
	gui_device.nnewton = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "hb1"));
	gui_device.hnewton = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pb1"));
	gui_device.lnewton = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "timedomain"));
	gui_device.time_on = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "voltageorlight"));
	gui_device.pulsevoltage = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "psun"));
	gui_device.Psun=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "startvoltage"));
	gui_device.Vstart=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "stopvoltage"));
	gui_device.Vstop=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "stepvoltage"));
	gui_device.Vstep=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "ddclamp"));
	gui_device.electrical_clamp=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "eclamp"));
	gui_device.thermal_clamp=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget) );
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "hclamp"));
	gui_device.thermal_clamp=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "dt"));
	gui_device.dt=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget))*1e-9;

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "onvalue"));
	gui_device.onvalue=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "offvalue"));
	gui_device.offvalue=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pulsestart"));
	gui_device.pulsestart=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "reprate"));
	gui_device.reprate=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pulselength"));
	gui_device.pulselen=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));



	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "electricalerror"));
	gui_device.min_cur_error=pow(10.0,-gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget)));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "tleft"));
	gui_device.Tll=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "tr"));
	gui_device.Tlr=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

save_config(gui.filename,&gui_device);
gtk_widget_hide(gui.configure);
}


void configure_hide (GtkObject *object, gpointer user_data)
{

	gtk_widget_hide(gui.configure);
}

void meshedit_show (GtkObject *object, gpointer user_data)
{
GtkWidget       *window=NULL;
do_mesh_edit (&gui_device,window);
}


void show_epitaxy_builder (GtkObject *object, gpointer user_data)
{
GtkWidget       *window=NULL;
do_editable_cells (&gui_device,window);
}


void show_iv (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - JV curve'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'iv.plot'\n");
	fflush(plot);
	fclose(plot);
}

void stopsimulation (GtkObject *object, gpointer user_data)
{
printdebug( "Halting simulation\n");
	cell.stop=TRUE;
}


void show_converge (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - Convergence'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'converge.plot'\n");
	fflush(plot);
	fclose(plot);
}

void showthermal (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - Thermal profile'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'thermal.plot'\n");
	fflush(plot);
	fclose(plot);
}

void showpulsetrainv (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - Time domain pulsing voltage'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'pulsetrainv.plot'\n");
	fflush(plot);
	fclose(plot);
}

void showpulsetrainl (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - Time domain pulsing light'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'pulsetrainl.plot'\n");
	fflush(plot);
	fclose(plot);
}

void open_sim_file (GtkObject *object, gpointer user_data)
{
GtkWidget *dialog;

dialog = gtk_file_chooser_dialog_new ("Open Simulation File",NULL,	GTK_FILE_CHOOSER_ACTION_OPEN,GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,NULL);

if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
{
	char *filename;
	gtk_widget_hide(dialog);
	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
	strcpy(gui.filename,filename);
	load_config(filename,&gui_device,TRUE);
}else
{
gtk_widget_destroy (dialog);
}
}

gboolean splash_hide_now(GtkObject *object, gpointer data)
{
    GtkWidget * window_10_sec = (GtkWidget *) data;
    gtk_widget_hide(window_10_sec);
    return(FALSE);
}

gboolean splash_hide(gpointer data){
    GtkWidget * window_10_sec = (GtkWidget *) data;
    gtk_widget_hide(window_10_sec);
    return(FALSE);
}


#endif

void update(struct device *in)
{
int i;
int band;
int bands=in->bands;
for (i=0;i<in->ymeshpoints;i++)
{

	for (band=0;band<bands;band++)
	{
		in->t[i][band]=in->Xi[i];
	}

	in->Ec[i]=-in->phi[i]-in->Xi[i];
	in->Ev[i]=-in->phi[i]-in->Xi[i]-in->Eg[i];
	in->R[i]=0.0;
	in->Nc[i]=2.0*pow(((2.0*pi*kb*in->Tl[i]*in->me[i]*m0)/(hp*hp)),1.5);
	in->Nv[i]=2.0*pow(((2.0*pi*kb*in->Tl[i]*in->mh[i]*m0)/(hp*hp)),1.5);
	for (band=0;band<bands;band++)
	{
	in->dn[i][band]=q*in->Nc[i]*exp((q*(in->Fn[i][band]-in->Ec[i]))/(kb*in->Tl[i]))/(kb*in->Tl[i]);
	in->dp[i][band]=q*in->Nv[i]*exp((q*(in->Ev[i]-in->Fp[i][band]))/(kb*in->Tl[i]))/(kb*in->Tl[i]);
	in->n[i][band]=in->Nc[i]*exp(((in->Fn[i][band]-in->Ec[i])*q)/(kb*in->Tl[i]));
	in->p[i][band]=in->Nv[i]*exp(((in->Ev[i]-in->Fp[i][band])*q)/(kb*in->Tl[i]));
	in->nlast[i]=in->n[i][band];
	in->plast[i]=in->p[i][band];
	}


	in->nt[i]=in->Nct[i]*exp(((in->Fnt[i]-in->Ec[i])*q)/(kb*in->Tl[i]));
	in->dnt[i]=q*in->Nct[i]*exp((q*(in->Fnt[i]-in->Ec[i]))/(kb*in->Tl[i]))/(kb*in->Tl[i]);

	in->pt[i]=in->Nvt[i]*exp(((in->Ev[i]-in->Fpt[i])*q)/(kb*in->Tl[i]));
	in->dpt[i]=q*in->Nvt[i]*exp((q*(in->Ev[i]-in->Fpt[i]))/(kb*in->Tl[i]))/(kb*in->Tl[i]);

}

}


void solve_all(struct device *in,int thermal,int exciton)
{
	if (exciton==TRUE)
	{
		int cont=TRUE;
		int ittr=0;
		do
		{
		solve_cur_thermal(in,thermal);
		solve_excite(in);
		if (((in->excite_conv==TRUE)&&(in->dd_conv==TRUE))||(ittr>10)) cont=FALSE;
		ittr++;
		}while(cont==TRUE);

	}else
	{

		solve_cur_thermal(in,thermal);
	}

}

void dialog_set_progress (double percent)
{
#ifdef makegui
if (gui.show==TRUE)
{
		printf ("percent=%e\n", percent);
		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(gui.progress), percent);
;

	quick_update();
}
#endif
}

void device_init(struct device *in)
{
	in->Voc=0.0;
	in->Jsc=0.0;
	in->FF=0.0;
	in->Pmax=0.0;
	in->Pmax_voltage=0.0;
	device_get_memory(in);

}

void find_n0(struct device *in)
{
dprintf("Finding n0\n");
double oldsun=in->Psun;
int bands=in->bands;
int i;
double oldv=in->Vapplied;
in->Vapplied=0;
in->Psun=0;
light_field(in,in->spectra);
solve_all(in,FALSE,in->external_exciton_solver);
int band;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
	in->ninit[i]=in->n[i][0];
	in->pinit[i]=in->p[i][0];
	}
}

in->n0=get_extracted_np(in);
in->Psun=oldsun;
in->Vapplied=oldv;
light_field(in,in->spectra);
dprintf("Found n0 %e\n",in->n0);
//getchar();
}

void quick_update()
{
#ifdef makegui
if (gui.show==TRUE)
{
	int i=0;
	while ((gtk_events_pending ())&&(i<100))
	{

		gtk_main_iteration ();
		i++;

	}
}
#endif
}

int run_simulation(char *name)
{

gen_dos_fd_gaus_fd();
char out[2000];
int i;
printf("SolarSim V0.1 License: This software is not open source and the source code or the executable may not be distributed.   I do not mind you using this code for your academic research, however, due to the large amount of time I have invested in the software I must insist that I am included as an author on any resulting publications.  rod@roderickmackenzie.eu\n");

cell.kTq=(300.0*kb/q);
system("mv rbi.dat rbi.old");
system("mv jv.dat jv.old");
system("mv charge.dat charge.old");
system("mv ivexternal.dat ivexternal.old");
printf("load config\n");
load_config(name,&cell,FALSE);
printf("dos init\n");
dos_init();
//load_dos(&cell,"./dosn_mb.dat","./dosp_mb.dat",1);
printf("load dos\n");
load_dos(&cell,"./dosn_gau.dat","./dosp_gau.dat",1);
printf("bands=%d\n",cell.bands);
device_init(&cell);

#ifdef dumpconverge
cell.converge = fopen("./converge.dat","w");
fclose(cell.converge);
#endif
cell.tconverge=fopen("./tconverge.dat","w");
fclose(cell.tconverge);

int band;
int bands=cell.bands;

int curlayer=0;
double end=cell.mat.l[0].height;

for (i=0;i<cell.ymeshpoints;i++)
{
	if (cell.ymesh[i]>=end)
	{
		curlayer++;
		end+=cell.mat.l[curlayer].height;
	}
	cell.ymatlayers[i]=curlayer;
	cell.Nad[i]=cell.mat.l[cell.ymatlayers[i]].Nad;

}
//getchar();
cell.ylen=0.0;
for (i=0;i<cell.mat.number;i++)
{
cell.ylen+=cell.mat.l[i].height;
}
	for (i=0;i<cell.ymeshpoints;i++)
	{
		cell.Tl[i]=cell.Tll+cell.ymesh[i]*(cell.Tlr-cell.Tll)/cell.ylen;
		cell.Te[i]=cell.Tll+cell.ymesh[i]*(cell.Tlr-cell.Tll)/cell.ylen;
		cell.Th[i]=cell.Tll+cell.ymesh[i]*(cell.Tlr-cell.Tll)/cell.ylen;
		cell.ex[i]=0.0;
		cell.Hex[i]=0.0;
		//if ((i>cell.ymeshpoints/2)&&(i<cell.ymeshpoints/2+10)) cell.Hex[i]=1e9;
		cell.epsilonr[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_epsilonr);
		cell.dostype[i]=(int)get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_dostype);

		cell.Eg[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_Eg);//+fabs(dos_get_Ev_edge(cell.dostype[i]))+fabs(dos_get_Ec_edge(cell.dostype[i]));
		//printf("Eg=%e\n",cell.Eg[i]);
		cell.Dex[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_Dex);

		cell.Xi[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_Xi);//-fabs(dos_get_Ec_edge(cell.dostype[i]));
		//printf("Xi=%e\n",cell.Xi[i]);
		//getchar();
//printf("edge---------------|%e %e\n",dos_get_Ev_edge(cell.dostype[i]),dos_get_Ec_edge(cell.dostype[i]));
//getchar();
		cell.Ec[i]=-cell.Xi[i];

		cell.Ev[i]=-cell.Xi[i]-cell.Eg[i];

		//printf("%d %e %e\n",i,cell.mun[i],cell.mup[i]);

		cell.me[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_me);
		cell.mh[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_mh);
		cell.Nc[i]=2.0*pow(((2.0*pi*kb*cell.Tl[i]*cell.me[i]*m0)/(hp*hp)),1.5);

		cell.Nv[i]=2.0*pow(((2.0*pi*kb*cell.Tl[i]*cell.mh[i]*m0)/(hp*hp)),1.5);
		cell.ni[i]=sqrt(cell.Nc[i]*cell.Nv[i])*exp(-cell.Eg[i]/(2.0*cell.kTq));

		for (band=0;band<bands;band++)
		{
		cell.mun[i][band]=get_mat_param_mobility(&(cell.mat.l[cell.ymatlayers[i]]),mat_mue,band);
		cell.mup[i][band]=get_mat_param_mobility(&(cell.mat.l[cell.ymatlayers[i]]),mat_muh,band);
		}

		cell.kf[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_kf);
		cell.a[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_a);
		cell.kl[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_kl);
		cell.ke[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_kl);
		cell.kh[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_kl);

		cell.Hl[i]=0.0;
		cell.He[i]=0.0;
		cell.Hh[i]=0.0;
		cell.Hasorb[i]=0.0;

		//Electron capture
		cell.tauncap[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_tauncap);
		cell.taunescape[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_taunescape);
		cell.Nct[i]=2.0*pow(((2.0*pi*kb*cell.Tl[i]*cell.me[i]*m0)/(hp*hp)),1.5);	//less dense than bulk states

		cell.taupcap[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_taupcap);
		cell.taupescape[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_taupescape);
		cell.Nvt[i]=2.0*pow(((2.0*pi*kb*cell.Tl[i]*cell.mh[i]*m0)/(hp*hp)),1.5);
		cell.A[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_A);
		cell.deltaE[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_deltaE);
		cell.capeff[i]=get_mat_param_poly(&(cell.mat.l[cell.ymatlayers[i]]),mat_capeff,0.0);

	}
//getchar();

	for (i=0;i<cell.ymeshpoints;i++)
	{
		cell.phi[i]=0.0;
		cell.R[i]=0.0;
		for (band=0;band<bands;band++)
		{
			cell.n[i][band]=0.0;
		}
	}


light_field(&cell,cell.spectra);



update(&cell);


cell.Vapplied=0.0;

get_initial(&cell);


solve_pos(&cell);

solver_get_memory(&cell);

if (cell.onlypos==TRUE)
{
	device_free(&cell);
	return 0;
}

plot_open(&cell);
int nnewton=cell.nnewton;
int hnewton=cell.hnewton;
	//getchar();
//cell.nnewton=FALSE;
//cell.hnewton=FALSE;

//cell.max_electrical_itt=30;
//solve_all(&cell,FALSE,cell.external_exciton_solver);
//solve_thermal(&cell);
cell.stop=FALSE;
cell.go_time=FALSE;
cell.nnewton=nnewton;
cell.hnewton=hnewton;




plot_now(&cell,"plot");


int IV=TRUE;
if (IV==TRUE)
{
FILE *iv;
iv=fopen("./jv.dat","w");
fclose(iv);
iv=fopen("./charge.dat","w");
fclose(iv);
iv=fopen("./ivexternal.dat","w");
fclose(iv);
iv=fopen("./avgmu.dat","w");
fclose(iv);
iv=fopen("./freetrapped.dat","w");
fclose(iv);


FILE *file_rbi;
file_rbi=fopen("./rbi.dat","a");
double den=1e21;
double offset=0.0;
double mu=0.0;
do
{
offset=get_top_from_n(den,300.0,0.1,1e27,1,dos_all);
mu=cell.alpha*get_n_mu(offset,300.0,0.1,1);
offset=get_top_from_p(den,300.0,0.1,1e27,1,dos_all);
mu+=cell.beta*get_p_mu(offset,300.0,0.1,1);
mu/=2.0;
double langevin=q*(mu)/(3.8*epsilon0);
fprintf (file_rbi,"%e %e\n",den,langevin);
den*=1.2;
}while(den<5e23);

fclose(file_rbi);

//exit(0);
/*{
printf("%lf %lf\n",cell.Vapplied,cell.Vr);

cell.Vapplied=cell.Vr;

solve_all(&cell,gothermal,cell.external_exciton_solver);
getchar();
}*/


find_n0(&cell);
dirty_fit(&cell);
//rbi_test(&cell);


draw_gaus(&cell);

if (cell.time_on==TRUE)
{

	sim_time(&cell);

} if (cell.do_ce==TRUE)
{
	sim_ce(&cell);
}
else
{
	sim_jv(&cell);

}


}else
{
	cell.Vapplied=0.2;
	solve_cur(&cell);
	printf("V=%lf current = %e mA, J= %e A/m^2\n",cell.Vapplied,get_I(&cell)/1e-3,get_J(&cell));
}
#ifdef makegui
dialog_set_progress (0.0);
#endif
device_free(&cell);

plot_close(&cell);
cell.FF=cell.Pmax/(cell.Jsc*cell.Voc);


sprintf(out,"Voc= %lf (V)\n",cell.Voc);
printdebug( out);
sprintf(out,"Jsc= %lf (A/m^2)\n",cell.Jsc);
printdebug( out);
sprintf(out,"Pmax= %lf (W/m^2)\n",cell.Pmax);
printdebug( out);
sprintf(out,"Voltage to get Pmax= %lf (V)\n",cell.Pmax_voltage);
printdebug( out);
sprintf(out,"FF= %lf\n",cell.FF*100.0);
printdebug( out);
sprintf(out,"Efficiency= %lf percent\n",mod(cell.Pmax/cell.Psun)*100.0);
printdebug( out);
dos_free();
solver_free_memory(&cell);

return 0;
}

struct device gui_device;


int scanarg( char* in[],int count,char * find)
{
int i;
for (i=0;i<count;i++)
{
if (strcmp(in[i],find)==0) return TRUE;
}
return FALSE;
}

int main (int argc, char *argv[])
{
cell.fast=FALSE;
//printf("%le\n",read_value(0,2));
//getchar();
//copy_file("",2,1e-2);
//exit(0);
//solve_bi();
#ifdef makegui
gui.show=FALSE;

strcpy(gui.filename,"./blom_device.inp");

if (scanarg( argv,argc,"--gui")==TRUE)
{

	load_config(gui.filename,&gui_device,TRUE);

	gtk_init (&argc, &argv);
	gui.show=TRUE;
	GdkPixbuf *window_icon;

	window_icon=gdk_pixbuf_new_from_file( "sun_icon.png", NULL);

	gtk_window_set_default_icon (window_icon);

	gui.builder = gtk_builder_new ();
	gtk_builder_add_from_file (gui.builder, "main.glade", NULL);
	gui.window = GTK_WIDGET (gtk_builder_get_object (gui.builder, "window1"));
	gtk_window_set_title(GTK_WINDOW(gui.window), "Solarsim");
	gui.progress = GTK_WIDGET (gtk_builder_get_object (gui.builder, "mainprogres"));
	gui.configure = GTK_WIDGET (gtk_builder_get_object (gui.builder, "configure"));
	gtk_window_set_title(GTK_WINDOW(gui.configure), "Solarsim - configure");
	gui.textview = GTK_WIDGET (gtk_builder_get_object (gui.builder, "textview"));
	printdebug( "Solarsim V 0.1\n");

	gtk_builder_connect_signals (gui.builder, NULL);

	gui.splash = GTK_WIDGET (gtk_builder_get_object (gui.builder, "splash"));
	g_timeout_add (2000, splash_hide, gui.splash);

	gtk_widget_show (gui.window);
	gtk_widget_show (gui.splash);


	gtk_main ();
}else
#endif

if (scanarg( argv,argc,"--tab")==TRUE)
{
dump_talbe();
exit(0);
}

if (scanarg( argv,argc,"--fit")==TRUE)
{
int docopy=TRUE;
FILE *file;
if ((file=fopen("./rsyncdir.inp","r"))==NULL)
{
	docopy=FALSE;
}

char path[1000];
char command[1000];

if (docopy==TRUE)
{
fscanf(file,"%s",path);
fclose(file);
sprintf(command,"rsync ./* %s\n",path);
}
	cell.fast=TRUE;
	do
	{
	fit_now();

	if (docopy==TRUE)
	{
		printf("Running command %s\n",command);
		system(command);
	}
	//getchar();
	}while(1);
}else
{
	run_simulation("blom_device.inp");
}
	return 0;
}

